﻿using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Request.Management;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.DTO.Management;
using WingInterfaceLibrary.DTO.Organization;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.User;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.DTO.ReportTemplate;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Result.UserFeature;
using System;
using WingInterfaceLibrary.DTO.Schedule;
using WingInterfaceLibrary.DTO.Common;

namespace WingInterfaceLibrary.Interface.DBInterface
{
    public interface IUserDBService
    {
        Task<string> InsertUserAsync(CreateUserInfoDBRequest request);
        Task<UserDTO> FindUserByNameAsync(string name);

        /// <summary>
        /// 根据OpenId查找用户
        /// </summary>
        /// <param name="name">微信OpenId</param>
        Task<UserDTO> FindUserByOpenIdAsync(string openId);

        /// <summary>
        /// 根据LabDongle查找用户
        /// </summary>
        /// <param name="name">LabDongle</param>
        Task<UserDTO> FindUserByLabDongleDBAsync(string labDongle);

        /// <summary>
        /// 绑定/解绑用户OpenId
        /// </summary>
        /// <param name="name">微信OpenId</param>
        Task<bool> SetUserOpenIdAsync(string userCode, string openId);
        Task<UserDTO> FindUserByCodeAsync(string code);
        Task<UserDTO> FindUserAsync(FindUserRequest request);
        Task<UserPasswordDTO> FindUserPasswordByCodeAsync(string code);
        Task<UserDTO> FindUserByPhoneAsync(string phone);
        Task<UserDTO> FindUserByEmailAsync(string email);
        Task<PageCollection<UserDTO>> FindUserPagesAsync(PageFilterRequest request);
        Task<bool> DeleteUserByCodeAsync(string code);
        Task<bool> UpdateUserAsync(string code, UserPasswordDTO data, string extensionData, bool canNull = false, bool isChangeHeadImg = true);
        Task<long> UpdateUsersAsync(UpdateUsersDBRequest request);
        Task<List<UserDTO>> FindUserListByRoleCodeAsync(string roleCode);
        Task<List<UserDTO>> FindUsersAsync(GetUsersDBRequest request);

        /// <summary>
        /// 根据机构codes获取用户列表
        /// </summary>
        /// <param name="OrganizationCodes"></param>
        /// <returns></returns>
        Task<List<UserDTO>> FindUserListByOrganizationCodesAsync(List<string> organizationCodes);

        /// <summary>
        /// 根据机构codes和角色获取用户列表
        /// </summary>
        /// <param name="OrganizationCodes"></param>
        /// <returns></returns>
        Task<List<UserDTO>> FindUserListByOrganizationCodesAndRolesAsync(FindUserListByOrganizationCodesAndRolesRequest request);

        /// <summary>
        /// 根据codes获取用户列表
        /// </summary>
        /// <param name="codes">用户codes</param>
        /// <returns></returns>
        Task<List<UserDTO>> FindUserListByUserCodesAsync(List<string> codes);

        /// <summary>
        /// 将设备绑定到用户中
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<long> BingDevicesToUsersAsync(BingDevicesToUsersRequest request);

        /// <summary>
        /// 验证用户密码是否正确
        /// </summary>
        /// <param name="UserCode"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        Task<bool> VerificationPasswordAsync(string UserCode, string password);

        /// <summary>
        /// 获取设备用户列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageResult<ManageDeviceUserDTO>> GetDeviceUserPagesAsync(GetDeviceUserPagesRequest request);

        /// <summary>
        /// 根据设备获取用户列表 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<List<UserDTO>> GetUsersByDeviceAsync(string deviceCode);

        /// <summary>
        /// 根据设备Codes获取用户列表 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<List<UserDTO>> GetUsersByDevicesAsync(List<string> deviceCodes);

        /// <summary>
        /// 查询机构用户列表（分页）
        /// </summary>
        /// <param name="allNeeds"></param>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageResult<ManageOrganizationUserDTO>> GetManageOrganizationUserPagesAsync(List<OrganizationDTO> allNeeds, GetOrganizationUserPagesRequest request);

        /// <summary>
        /// 根据组织、岗位身份、职称查询用户列表信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<List<UserDTO>> GetUserListAsync(GetUserListDBRequest request);

        /// <summary>
        /// 根据组织、岗位身份、职称查询用户列表信息（分页）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageResult<UserDTO>> GetUserListPagesDBAsync(GetUserListPagesDBRequest request);

        /// <summary>
        /// 查找用户列表（分页）自定义条件
        /// </summary>
        /// <param name="request">分页信息</param>
        /// <returns></returns>
        Task<PageCollection<UserDTO>> FindUserPagesCustomAsync(PageFilterRequest request);

        /// <summary>
        /// 根据用户和医院名称集合查询用户列表信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageCollection<UserDTO>> FindUserPagesByOrgsAsync(FindUserPageByOrgsRequest request);

        /// <summary>
        /// 根据组织编码查找用户
        /// </summary>
        /// <param name="organizationCode"></param>
        /// <returns></returns>
        Task<List<UserDTO>> FindUsersByOrganizationCodeAsync(string organizationCode);

        /// <summary>
        /// 删除病人
        /// </summary>
        /// <returns></returns>
        Task<bool> DeletePatientsByUserCodeAsync(DeletePatientsDBRequest request);

        /// <summary>
        /// 设置用户默认词条库编码
        /// </summary>
        /// <param name="userCode">用户编码</param>
        /// <param name="thesaurusCode">默认词条库编码</param>
        /// <param name="languageCode">语言</param>
        /// <returns></returns>
        Task<bool> SetUserDefaultThesaurusCodeAsync(string userCode, string thesaurusCode, string languageCode = "en-US");

        /// <summary>
        /// 获取用户默认词条库编码
        /// </summary>
        /// <param name="userCode">用户编码</param>
        /// <returns>词条库编码</returns>
        Task<string> GetUserDefaultThesaurusCodeAsync(string userCode, string language = "en-US");

        /// <summary>
        /// 获取用户默认词条库信息
        /// </summary>
        /// <param name="userCode">用户编码</param>
        /// <param name="language">语言编码</param>
        /// <returns></returns>
        Task<ThesaurusAllDTO> GetUserDefaultThesaurusContentAsync(string userCode, string language = "en-US");

        /// <summary>
        /// 设置用户默认报告模板编码
        /// </summary>
        /// <param name="userCode">用户编码</param>
        /// <param name="thesaurusCode">默认报告模板编码</param>
        /// <param name="languageCode">语言编码</param>
        /// <returns></returns>
        Task<bool> SetUserDefaultReportTemplateCodeAsync(string userCode, string thesaurusCode, string languageCode = "en-US");

        /// <summary>
        /// 获取用户默认报告模板编码
        /// </summary>
        /// <param name="userCode">用户编码</param>
        /// <param name="language">语言编码</param>
        /// <returns>报告模板编码</returns>
        Task<string> GetUserDefaultReportTemplateCodeAsync(string userCode, string language = "en-US");

        /// <summary>
        /// 获取用户默认报告模板信息
        /// </summary>
        /// <param name="userCode">用户编码</param>
        /// <param name="language">语言编码</param>
        /// <returns></returns>
        Task<ReportTemplateDTO> GetUserDefaultReportTemplateContentAsync(string userCode, string language = "en-US");

        /// <summary>查询专家助理绑定的所有专家</summary>
        /// <returns></returns>
        Task<IList<UserDTO>> GetBindExpertsAsync(string assistentCode);


        /// <summary>
        /// 根据codes获取用户列表
        /// </summary>
        /// <param name="codes"></param>
        /// <returns></returns>
        Task<List<UserPasswordDTO>> FindUsersByUserCodesAsync(List<string> codes);

        /// <summary>
        /// 更新用户登录锁定次数
        /// </summary>
        /// <param name="code"></param>
        /// <param name="loginLockInfoDTO"></param>
        /// <returns></returns>
        Task<bool> UpdateUserLoginLockInfoAsync(string code, LoginLockInfoDTO loginLockInfoDTO);
        /// <summary>
        /// 更新用户密码 
        /// </summary>
        /// <param name="code"></param>
        /// <param name="password"></param>
        /// <param name="pwdRecords"></param>
        /// <param name="updateTime"></param>
        /// <returns></returns>
        Task<bool> UpdateUserPasswordAsync(string code, string password, IList<string> pwdRecords = null, DateTime? updateTime = null);
        /// <summary>
        /// 更新用户登录信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> UpdateUserLoginInfoAsync(UpdateUserLoginInfoDBRequest request);

        /// <summary>
        /// 获取最后新增的用户的创建时间
        /// </summary>
        /// <returns></returns>
        Task<DateTime> GetLastUserCreateTime();


        Task<string> AddScheduleDBAsync(AddScheduleDBRequest request);

        Task<bool> UpdateScheduleDBAsync(UpdateScheduleDBRequest request);

        Task<bool> AddScheduleUsersDBAsync(AddScheduleUsersDBRequest request);

        Task<bool> UpdateScheduleStateDBAsync(UpdateScheduleStateDBRequest request);

        Task<bool> DeleteScheduleDBAsync(string code);

        Task<ScheduleDTO> GetScheduleDBByCodeAsync(string code);

        Task<IList<ScheduleDTO>> GetScheduleDBAsync(GetScheduleDBRequest request);

        Task<ScheduleDTO> GetScheduleDBByRelevanceCodeAsync(string code);

        /// <summary>
        /// 查找有效用户总数量
        /// </summary>
        /// <returns></returns>
        Task<long> GetActiveUsersCount();

        /// <summary>
        /// 通过多个角色获取用户信息
        /// </summary>
        /// <param name="roleCodes"></param>
        /// <returns></returns>
        Task<List<UserDTO>> FindUserListByRoleCodesAsync(IList<string> roleCodes);

        /// <summary>
        /// 更新第三方关联信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> UpdateAssociatedInfosAsync(UpdateAssociatedInfosDBRequest request);

        /// <summary>
        /// 修改用户急诊设备
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> UpdateEmergencyDeviceCodeDBAsync(UpdateEmergencyDeviceCodeDBRequest request);

        /// <summary>
        /// 修改用户急诊配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> UpdateEmergencyInfoDBAsync(UpdateEmergencyInfoDBRequest request);

        /// <summary>
        /// 清楚第三方关联信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> ClearAssociatedInfosAsync(ClearAssociatedInfosDBRequest request);

        /// <summary>
        /// 批量新增用户数据
        /// </summary>
        /// <param name="request">登录token信息</param>
        /// <returns>是否成功</returns>
        Task<bool> BatchInsertUserDataDBAsync(BatchInsertUserRequest request);

        /// <summary>
        /// 清洗有医院的用户数据
        /// </summary>
        /// <param name="request">登录token信息</param>
        /// <returns>是否成功</returns>
        Task<bool> BatchUpdateUserOrganizationDBAsync(BatchInsertUserRequest request);

        /// <summary>
        /// 获取待迁移时间
        /// </summary>
        /// <returns>是否成功</returns>
        Task<MigrateRecordDTO> QueryMigrateTimeDBAsync(bool isInitMigreate, MigrateRecordType type = MigrateRecordType.Default);

        /// <summary>
        /// 根据用户编码查询用户
        /// </summary>
        /// <param name="code">用户编码</param>
        Task<UserDTO> FindUserDTOByCodeAsync(string code, bool isNeedExtendInfo);

        /// <summary>
        /// 根据绑定处理和急诊医生查询用户列表信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<List<UserDTO>> GetUserListByBindUserCodeAsync(string userCode);

        /// <summary>
        /// 清洗数据-修改用户绑定设备机构，以及绑定机构的机构负责人对应的设备和机构数据；
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        Task<long> ModifyUserBindDeviceOrganizationsDBAsync(ModifyUserBindDeviceOrganizationRequest request);

        /// <summary>
        /// 清洗数据-将机构数据为空，且用户角色不为空的用户数据清洗到vinno_fis平台下面
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        Task<bool> ModifyUserSpecialOrganizationInfoDBAsycn(ModifyUserSpecialOrganizationRequest request);

        /// <summary>
        /// 通用用户注销
        /// </summary>
        /// <param name="request">注销请求实体</param>
        /// <returns>是否成功</returns>
        Task<bool> CommonUserLogOffDBAsync(UserPasswordDTO request);

        /// <summary>
        /// 更新用户角色为空的，调整为默认普通用户角色
        /// </summary>
        /// <returns>是否成功</returns>
        Task<bool> UpdateUserBindRoleDBAsync(ModifyUserBindDeviceOrganizationRequest request);
        
        /// <summary>
        /// 修改/删除个人机构，清空个人绑定数据
        /// </summary>
        /// <returns>是否成功</returns>
        Task<bool> ClearPersonDefaultTemplateDBAsync(string userCode, string language);

        /// <summary>
        /// 保存用户权限列表codes
        /// </summary>
        /// <returns>数据列表</returns>
        Task<bool> SaveUserFeatureCodesDBAsync(SaveUserFeatureInfoRequest request);

        /// <summary>
        /// 分页获取资源计费信息列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageResult<ChargeResourcePackageDTO>> GetChargeResourcePackagePagesDBAsync(ChargeResourcePackagePageRequest request);

        /// <summary>
        /// 根据Code获取资源计费信息数据
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        Task<ChargeResourcePackageDTO> GetChargeResourcePackageByCodeDBAsync(string code);

        /// <summary>
        /// 保存资源计费信息数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> SaveChargeResourcePackageDBAsync(SaveChargeResourcePackageRequest request);

        /// <summary>
        /// 删除资源计费信息数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> DeleteChargeResourcePackageDBAsync(DeleteChargeResourcePackageRequest request);

        /// <summary>
        /// 根据用户名查找用户
        /// </summary>
        /// <param name="name">用户名</param>
        Task<UserDTO> FindUserByCommonPlatformUserIdAsync(string commonPlatformUserId);
    }
}
