﻿using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.DTO.Rank;
using WingInterfaceLibrary.Request.User;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.AIDiagnosis;
using WingInterfaceLibrary.Result.AIDiagnosis;
using WingInterfaceLibrary.DTO.Comment;
using WingInterfaceLibrary.Request.RemedicalAISelected;
using WingInterfaceLibrary.DTO.Record;

namespace WingInterfaceLibrary.Interface
{
    /// <summary>
    /// AI诊断服务接口
    /// </summary>
    /// <docFirstDirectory>AI诊断API</docFirstDirectory>
    /// <docSecondDirectory>AI诊断模块</docSecondDirectory>
    public interface IAIDiagnosisService
    {
        /// <summary>
        /// 查询杏聆荟支持的AI模块 
        /// </summary>
        /// <param name="request">查询杏聆荟支持的AI模块请求实体</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<IList<string>> FindDiagnosisModulesAsync(TokenRequest request);

        /// <summary>
        /// 图像诊断
        /// </summary>
        /// <param name="request">图像诊断请求实体</param>
        /// <returns>图像诊断结果</returns>
        /// <show>false</show>
        Task<DiagnosisImageResult> DiagnosisImageAsync(DiagnosisImageRequest request);

        /// <summary>
        /// 根据结果图像诊断
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<DiagnosisImageResult> GetDiagnosisImageResultByAIResultAsync(GetDiagnosisImageResultByAIResultRequest request);

        /// <summary>
        /// 根据结果生成颈动脉诊断
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<CarotidResultDTO> GetCarotidResultAsync(GetCarotidResultRequest request);

        /// <summary>
        /// 根据颈动脉体数据生成3D 6面图像集合
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<CreateCarotidSurfaceImagesResult> CreateCarotidSurfaceImagesAsync(CreateCarotidSurfaceImagesRequest request);

        /// <summary>
        /// 生成AI报告
        /// </summary>
        /// <param name="request">生成AI报告请求实体</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<DiagnosisReportResult> DiagnosisReportAsync(DiagnosisReportRequest request);

        /// <summary>
        /// 查询AI诊断结论
        /// </summary>
        /// <param name="request">查询AI诊断结论请求实体</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<GetDiagnosisConclusionResult> GetDiagnosisConclusionAsync(GetDiagnosisConclusionRequest request);

        /// <summary>
        /// 查询AI相关枚举集合
        /// </summary>
        /// <param name="request">查询AI相关枚举集合请求实体</param>
        /// <returns>AI相关枚举集合</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<GetDiagnosisEnumItemsResult> GetDiagnosisEnumItemsAsync(GetDiagnosisEnumItemsRequest request);

        /// <summary>
        /// 查询病灶轮廓的关键点 
        /// </summary>
        /// <param name="request">查询病灶轮廓的关键点请求实体</param>
        /// <returns>关键点集合</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<List<DiagnosisKeyPointDTO>> GetKeyPointsOfContourAsync(GetKeyPointsOfContourRequest request);

        /// <summary>
        /// 移动光标，查询受影响关键点 
        /// </summary>
        /// <param name="request">移动光标，查询受影响关键点请求实体</param>
        /// <returns>受影响关键点下标</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<List<int>> AffectedKeyPointsByDragActionAsync(AffectedKeyPointsByDragActionRequest request);

        /// <summary>
        /// 拖动光标，查询所有轮廓点和关键点
        /// </summary>
        /// <param name="request">拖动光标，查询所有轮廓点和关键点请求实体</param>
        /// <returns>所有轮廓点和关键点</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<ContourAndKeyPointsAfterDragResult> ContourAndKeyPointsAfterDragAsync(ContourAndKeyPointsAfterDragRequest request);

        /// <summary>
        /// 画轮廓模式，查询鼠标位置到轮廓线的最短距离
        /// </summary>
        /// <param name="request">画轮廓模式，查询鼠标位置到轮廓线的最短距离请求实体</param>
        /// <returns>鼠标离轮廓线的距离，高亮点的下标</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<MinimumDistanceToContourPointsResult> MinimumDistanceToContourPointsAsync(MinimumDistanceToContourPointsRequest request);

        /// <summary>
        /// 画轮廓模式，合并新旧轮廓
        /// </summary>
        /// <param name="request">画轮廓模式，查询鼠标位置到轮廓线的最短距离请求实体</param>
        /// <returns>合并后的完整轮廓线，合并后的横纵径尺寸结果</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<ContourMergeResult> ContourMergeAsync(ContourMergeRequest request);

        /// <summary>
        /// URM分析
        /// </summary>
        /// <param name="request">URM分析请求实体</param>
        /// <returns>URM分析分析结果</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<URMAnalysisResult> URMAnalysisAsync(URMAnalysisRequest request);

        /// <summary>
        /// URM分析V2
        /// </summary>
        /// <param name="request">URM分析V2请求实体</param>
        /// <returns>URM分析分析结果V2</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<URMAnalysisV2Result> URMAnalysisV2Async(URMAnalysisV2Request request);

        /// <summary>
        /// URM后处理-预加载
        /// </summary>
        /// <param name="request">URM后处理请求实体</param>
        /// <returns>URM后处理结果</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<bool> LoadURMProcessAsync(URMProcessRequest request);

        /// <summary>
        /// URM后处理
        /// </summary>
        /// <param name="request">URM后处理请求实体</param>
        /// <returns>URM后处理结果</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<URMProcessResult> URMProcessAsync(URMProcessRequest request);

        /// <summary>
        /// 获取URMColorbar
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<URMColorbarResult> GetURMColorbarAsync(URMColorbarRequest request);

        /// <summary>
        /// URM视频后处理-预加载
        /// </summary>
        /// <param name="request">URM视频后处理请求实体</param>
        /// <returns>URM视频后处理结果</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<bool> LoadURMVideoProcessAsync(URMVideoProcessRequest request);

        /// <summary>
        /// URM视频后处理
        /// </summary>
        /// <param name="request">URM视频后处理请求实体</param>
        /// <returns>URM视频后处理结果</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<URMVideoProcessResult> URMVideoProcessAsync(URMVideoProcessRequest request);

        /// <summary>
        /// URM测量-预加载
        /// </summary>
        /// <param name="request">URM测量后处理测量请求实体</param>
        /// <returns>URM测量后处理结果</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<bool> LoadURMMeasureProcessAsync(URMMeasureProcessRequest request);

        /// <summary>
        /// URM测量
        /// </summary>
        /// <param name="request">URM测量测量请求实体</param>
        /// <returns>URM测量结果</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<URMMeasureProcessResult> URMMeasureProcessAsync(URMMeasureProcessRequest request);

        /// <summary>
        /// URM生成后处理视频
        /// </summary>
        /// <param name="request">URM生成后处理视频请求实体</param>
        /// <returns>URM生成后处理视频结果</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<URMGenerateVideoResult> URMGenerateVideoAsync(URMGenerateVideoRequest request);

        /// <summary>
        /// 小鼠频谱包络
        /// </summary>
        /// <param name="request">小鼠频谱包络请求实体</param>
        /// <returns>小鼠频谱包络结果</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<VetAutoTraceImageResult> VetAutoTraceImageAsync(VetAutoTraceImageRequest request);

        /// <summary>
        /// 远程图像区域缩放
        /// </summary>
        /// <param name="request">远程图像区域缩放请求实体</param>
        /// <returns>远程图像区域缩放结果</returns>
        /// <value></value>
        /// <errorCodes></errorCodes>
        Task<VidImageZoomResult> VidImageZoomAsync(VidImageZoomRequest request);
    }
}
