﻿using System;
using System.Collections.Generic;
using WingInterfaceLibrary.Enum;
using System.Threading.Tasks;
using WingInterfaceLibrary.Request.Authentication;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Result;

namespace WingInterfaceLibrary.Interface
{
    /// <summary>
    /// 权鉴 服务
    /// </summary>
    public interface IAuthenticationService
    {
        /// <summary>
        /// 获取权限验证
        /// </summary>
        /// <param name="request">验签平台类型</param>
        /// <returns>验证后生成的验签信息</returns>
        /// <value></value>
        /// <remarks>POST</remarks>
        /// <show>false</show>
        /// <errorCodes>FileNameIsEmpty,ExceptionError</errorCodes>
        Task<string> GetAuthorizationAsync(AuthenticationRequest request);

        /// <summary>
        /// 获取Vod权限验证
        /// </summary>
        /// <param name="request">Token对象</param>
        /// <returns>生成Vod的验签信息</returns>
        /// <remarks>POST</remarks>
        /// <errorCodes>ExceptionError</errorCodes>
        Task<string> GetVodAuthorizationAsync(TokenRequest request);

        Task<string> GetServerDefaultTokenAsync();

        /// <summary>
        /// Apply a token for client（Internal Api used on server）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<TokenDTO> ApplyTokenAsync(ApplyTokenRequest request);

        /// <summary>
        /// get tokendto
        /// </summary>
        /// <param name="request">Token string</param>
        /// <returns></returns>
        Task<TokenDTO> GetTokenAsync(TokenRequest request);

        /// <summary>
        /// Validate the token from client（Internal Api used on server）
        /// </summary>
        /// <param name="request">Token string</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<ValidateTokenResult> ValidateTokenAsync(ValidateTokenRequest request);

        /// <summary>
        /// Get tokens with specified client Id（Internal Api used on server）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<IList<TokenDTO>> GetTokensWithClientIdAsync(GetTokensWithClientIdRequest request);

        /// <summary>
        /// Get tokens with specified client ids
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<IList<TokenDTO>> GetTokenWithClientIdsAsync(GetTokenWithClientIdsRequest request);

        /// <summary>
        /// Get tokens with specified token values
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<IList<TokenDTO>> GetTokenWithValuesAsync(GetTokenWithValuesRequest request);

        /// <summary>
        /// Set token online state
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<bool> SetOnlineStateAsync(SetOnlineStateRequest request);

        /// <summary>
        /// Set token ip address
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<bool> SetIPAddressAsync(SetIPAddressRequest request);

        /// <summary>
        /// Log off from token manager（Internal Api used on server）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<bool> LogOffAsync(TokenRequest request);

        /// <summary>
        /// Get paged online accounts（Internal Api used on server）
        /// </summary>
        /// <param name="request">The request parameters</param>
        /// <returns>The online account list</returns>
        /// <show>false</show>
        Task<PageResult<TokenDTO>> GetPagedTokensAsync(GetPagedTokensOnlineRequest request);

        Task<bool> RefreshBlackInfosAsync(TokenRequest request);

        /// <summary>
        /// Set token install version address
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> SetTokenInstallVersionAsync(SetTokenInstallVersionRequest request);
    }

    /// <summary>
    /// Server token, just used in server
    /// </summary>
    public class TokenDTO
    {

        public TokenDTO()
        {

        }

        // /// <summary>
        // /// token dbCode
        // /// </summary>
        // /// <value></value>
        // public string Code { get; set; }

        /// <summary>
        /// The token version
        /// </summary>
        /// <value></value>
        public int Version { get; set; }
        /// <summary>
        /// The token string value
        /// </summary>
        /// <value></value>        
        public string Code { get; set; }

        /// <summary>
        /// The Account Type of logined client
        /// </summary>
        /// <value></value>
        public AccountType AccountType { get; set; }

        /// <summary>
        /// The Account name
        /// </summary>
        /// <value></value>
        public string AccountName { get; set; } = string.Empty;
        /// <summary>
        /// The plateform the account logined
        /// </summary>
        /// <value></value>
        public Platform Platform { get; set; }

        /// <summary>
        /// The login source of account
        /// </summary>
        /// <value></value>
        public LoginSource LoginSource { get; set; }

        /// <summary>
        /// The Client Id(User code , admin code or device code)
        /// </summary>
        /// <value></value>
        public string ClientId { get; set; }

        /// <summary>
        /// The Login server url
        /// </summary>
        /// <value></value>
        public string LoginServer { get; set; }

        /// <summary>
        /// Token create time
        /// </summary>
        /// <value></value>
        public DateTime CreateTime { get; set; }

        /// <summary>
        /// Token expiretion time
        /// </summary>
        /// <value></value>
        public DateTime Expiration { get; set; }

        /// <summary>
        /// The client ip address
        /// </summary>
        /// <value></value>
        public long IpValue { get; set; }

        /// <summary>
        /// The token online state
        /// </summary>
        /// <value></value>
        public bool IsOnline { get; set; }

        /// <summary>
        /// 安装版本
        /// </summary>
        /// <value></value>
        public string InstallVersion { get; set; }

        /// <summary>
        /// 是否老平台
        /// </summary>
        /// <value></value>
        public bool IsOldPlatform { get; set; }

        /// <summary>
        /// 老超声机连接方式
        /// </summary>
        /// <value></value>
        public ProxyTypeEnum ProxyType { get; set; }

        public TokenDTO(int version, string value, AccountType accountType, Platform platform, LoginSource loginSource, string clientId, string loginServer, int timeoutMinutes, long IpValue, string installVersion, bool isOldPlatform = false, ProxyTypeEnum proxyType = ProxyTypeEnum.Old)
        {
            this.Version = version;
            this.Code = value;
            this.AccountType = accountType;
            this.Platform = platform;
            this.LoginSource = loginSource;
            this.AccountType = accountType;
            this.ClientId = clientId;
            this.LoginServer = loginServer;
            this.CreateTime = DateTime.UtcNow;
            this.Expiration = CreateTime.AddMinutes(timeoutMinutes);
            this.IpValue = IpValue;
            this.InstallVersion = installVersion;
            this.IsOldPlatform = isOldPlatform;
            this.ProxyType = ProxyType;
        }

        public override string ToString()
        {
            return $"Token:{Code}|AccountType:{AccountType}|AccountName:{AccountName}|Platform:{Platform}|LoginSource:{LoginSource}|ClientId:{ClientId}|LoginServer:{LoginServer}|installVersion:{InstallVersion}|isOldPlatform{IsOldPlatform}|proxyType:{ProxyType}";
        }
    }

}