﻿using System.Threading.Tasks;
using WingInterfaceLibrary.Request.Device;
using WingInterfaceLibrary.DTO.Device;
using System.Collections.Generic;
using WingInterfaceLibrary.DTO.Dictionary;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.DTO.Management;
using WingInterfaceLibrary.DTO.DiagnosisModule;
using WingInterfaceLibrary.LiveConsultation;
using WingInterfaceLibrary.Result.Languge;
using WingInterfaceLibrary.Request.Examine;
using WingInterfaceLibrary.Request.Remote;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.Result.Device;
using WingInterfaceLibrary.DTO.RemoteParameters;
using WingInterfaceLibrary.Request.Comment;
using WingInterfaceLibrary.DTO.Common;

namespace WingInterfaceLibrary.Interface
{
    /// <summary>
    /// 设备信息服务接口
    /// </summary>
    /// <docFirstDirectory>客户端API</docFirstDirectory>
    /// <docSecondDirectory>设备信息接口</docSecondDirectory>
    public interface IDeviceService
    {
        /// <summary>
        /// 设备心跳
        /// </summary>
        /// <param name="request">设备心跳请求实体</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        Task<bool> HeartRateAsync(TokenRequest request);

        Task<DeviceInfoDTO> CreateDeviceInfoAsync(CreateDeviceRequest request);

        /// <summary>
        /// 根据设备code获取设备信息
        /// </summary>
        /// <param name="request">设备code请求实体</param>
        /// <returns>返回查询到的设备对象信息</returns>
        /// <errorCodes>DeviceCodeEmpty,NoExistDevice</errorCodes>
        Task<DeviceExtendInfoDTO> GetDeviceInfoAsync(GetDeviceRequest request);

        /// <summary>
        /// 根据名称查询设备信息
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>设备信息</returns>
        Task<DeviceInfoDTO> FindDeviceInfoByNameAsync(FindDeviceInfoByNameRequest request);

        /// <summary>
        /// 根据设备动态唯一码获取设备信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <errorCodes>ShortCodeEmpty,NoExistDevice</errorCodes>
        Task<DeviceExtendInfoDTO> GetDeviceByShortCodeAsync(GetDeviceByShortCodeRequest request);

        Task<PageCollection<DeviceInfoDTO>> GetDeviceInfoPageAsync(PageFilterRequest request);

        /// <summary>
        /// 绑定设备
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>UserNotExistError,ShortCodeEmpty,DeviceNotExist,HasAddedDevice,DeviceNameEmpty,TooLongCharacter,OrganizationCodeEmpty,OrganizationNotExist</errorCodes>
        Task<bool> BindDeviceAsync(BindDeviceRequest request);

        /// <summary>
        /// 修改设备信息
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>DeviceNotExist,DeviceNameEmpty,DeviceDescriptionEmpty</errorCodes>
        Task<bool> ModifyDeviceAsync(ModifyDeviceRequest request);

         /// <summary>
        /// 修改设备全部信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<bool> UpdateDeviceAsync(UpdateDeviceRequest request);

        /// <summary>
        /// 创建设备类型、设备型号字典项
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>字典编码</returns>
        /// <value>DictionaryDO_2022030202525405wUxJ</value>
        Task<string> CreateDictionaryItemAsync(CreateDictionaryItemRequest request);

        /// <summary>
        /// 查找设备型号所有字典项
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns>设备型号列表</returns>
        Task<List<DictionaryDTO>> FindDeviceModelItemsAsync(FindDeviceModelItemsRequest request);

        /// <summary>
        /// 查找设备类型所有字典项
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns>设备类型列表</returns>
        Task<List<DictionaryDTO>> FindDeviceTypeItemsAsync(FindDeviceTypeItemsRequest request);
        /// <summary>
        /// 增加分享设备关联用户
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>DeviceCodeIsEmpty,UserCodeEmptyError,NoExistDevice,UserNotExistError</errorCodes>
        Task<bool> CreateShareDeviceToUserAsync(CreateShareDeviceToUserRequest request);
        /// <summary>
        /// 删除分享设备关联用户
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>DeviceCodeIsEmpty,UserCodeEmptyError,NoExistDevice</errorCodes>
        Task<bool> DeleteShareDeviceToUserAsync(DeleteShareDeviceToUserRequest request);

        /// <summary>
        /// 机构设备移除
        /// </summary>
        /// <param name="request">移除设备请求实体</param>
        /// <returns>是否移除成功：true-成功；false-失败</returns>
        /// <value>true</value>
        /// <errorCodes>DeviceCodeOrOrganizationCodeEmpty,DeviceNotExist</errorCodes>
        Task<bool> RemoveDeviceRelevancyAsync(RemoveDeviceRelevancyRequest request);

        /// <summary>
        /// 查询个人名下所有设备列表接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns>返回设备信息列表</returns>
        /// <errorCodes>UserNotExistError,OrganizationNotExist</errorCodes>
        Task<PageCollection<DeviceExtendInfoDTO>> GetPersonDeviceListAsync(GetPersonDeviceRequest request);

        /// <summary>
        /// 查询个人名下本机构所有设备列表接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns>返回设备信息列表</returns>
        /// <errorCodes>UserNotExistError,OrganizationNotExist</errorCodes>
        Task<PageCollection<DeviceExtendInfoDTO>> GetPersonOrgDeviceListAsync(GetPersonDeviceRequest request);

        /// <summary>
        /// 查询设备列表（助理查专家的如果没有专家则查机构的）
        /// </summary>
        /// <param name="request">查询设备列表请求实体</param>
        /// <returns>返回设备信息列表</returns>
        Task<PageCollection<DeviceInfoDTO>> GetDeviceListByPersonRoleAsync(GetPersonRoleDeviceRequest request);

        /// <summary>
        /// 根据组织编码获取所属全部设备信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<List<DeviceExtendInfoDTO>> FindDevicesByOrganizationCodeAsync(FindDevicesByOrganizationCodeRequest request);

        /// <summary>
        /// 查询个人名下所有设备下拉列表接口
        /// </summary>
        /// <param name="request">签名令牌请求信息</param>
        /// <returns>返回设备下拉信息列表</returns>
        /// <errorCodes>UserNotExistError,OrganizationNotExist</errorCodes>
        Task<List<SelectItemDTO>> GetPersonDeviceDropdownListAsync(TokenRequest request);

        /// <summary>
        /// 查询获取服务器配置信息接口
        /// </summary>
        /// <param name="request">Token验证</param>
        /// <returns>服务器配置信息结果:key:字段，value:数据值，复杂类型可为json</returns>        
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>设备信息接口</docSecondDirectory>
        /// <errorCodes>NoExistDevice</errorCodes>
        Task<DeviceServerSettingResult> QueryServerConfigAsync(TokenRequest request);

        /// <summary>
        /// 添加设备到新的医院请求接口
        /// </summary>
        /// <param name="request">添加设备到新医院中请求实体</param>
        /// <returns>是否成功</returns>
        /// <errorCodes>UniqueCodeIsEmpty,NoExistDevice,OrganizationNotExist,UserNotExistError,NotOrgAdmin</errorCodes>
        Task<bool> AddDeviceToOrgAsync(AddDeviceToOrgRequest request);

        /// <summary>
        /// 获取设备绑定用户codes
        /// </summary>
        /// <param name="request">获取设备绑定用户codes请求实体</param>
        /// <returns>"test01","test02"</returns>
        /// <value>"test01","test02"</value>
        /// <errorCodes>DeviceCodeIsEmpty,NoExistDevice</errorCodes>
        Task<List<string>> GetDeviceBindUsersCodesAsync(GetDeviceRequest request);

        /// <summary>
        /// 查询设备AI应用的状态
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns>AI应用集合</returns>
        /// <errorCodes>DeviceCodeIsEmpty</errorCodes>
        Task<List<DiagnosisModuleDTO>> FindDeviceDiagnosisModulesAsync(FindDeviceDiagnosisModulesRequest request);

        /// <summary>
        /// 修改设备AI应用启用状态
        /// </summary>
        /// <param name="request">修改设备AI应用启用状态请求实体</param>
        /// <returns>是否成功</returns>
        /// <errorCodes>DeviceCodeIsEmpty,DeviceAIDiagnosisModule</errorCodes>
        Task<bool> ModifyDeviceDiagnosisModuleStateAsync(ModifyDeviceDiagnosisModuleStateRequest request);

        /// <summary>
        /// 上报设备画面尺寸相关信息
        /// </summary>
        /// <param name="request"></param>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>实时会诊服务接口</docSecondDirectory>   
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <returns></returns>
        Task<ReportVideoDeviceInfoResult> ReportVideoDeviceInfoAsync(ReportVideoDeviceInfoRequest request);

        /// <summary>
        /// 查找有效设备总数量
        /// </summary>
        /// <returns></returns>
        Task<long> GetActiveDeviceCount();

        /// <summary>
        /// 设备发送调参数据
        /// </summary>
        /// <param name="request">调参数据请求</param>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>实时会诊服务接口</docSecondDirectory>   
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <returns></returns>
        Task<bool> SendControlParameterByDeviceAsync(SendControlParameterByDeviceRequest request);

        /// <summary>
        /// 用户进入设备直播房间
        /// </summary>
        /// <param name="request">用户进入设备直播房间请求实体</param>
        /// <errorCodes>DeviceNotOnline,DeviceLiveClosed</errorCodes>
        /// <returns></returns>
        Task<JoinDeviceLiveRoomResult> JoinDeviceLiveRoomAsync(JoinDeviceLiveRoomRequest request);

        /// <summary>
        /// 用户离开设备直播房间
        /// </summary>
        /// <param name="request">用户离开设备直播房间请求实体</param>
        /// <errorCodes></errorCodes>
        /// <returns></returns>
        Task<bool> LeaveDeviceLiveRoomAsync(LeaveDeviceLiveRoomRequest request);

        /// <summary>
        /// 上报观看直播状态
        /// </summary>
        /// <param name="request">上报观看直播状态请求实体</param>
        /// <errorCodes></errorCodes>
        /// <returns></returns>
        Task<bool> ReportLiveViewStateAsync(ReportLiveViewStateRequest request);

        /// <summary>
        /// 生成设备直播分享地址
        /// </summary>
        /// <param name="request">生成设备直播分享地址请求实体</param>
        /// <errorCodes>DeviceLiveClosed</errorCodes>
        /// <returns></returns>
        Task<CreateLiveShareInfoResult> CreateLiveShareInfoAsync(CreateLiveShareInfoRequest request);

        /// <summary>
        /// 用户进入设备直播房间（无token）
        /// </summary>
        /// <param name="request">用户进入设备直播房间请求实体</param>
        /// <errorCodes>ShareCodeIsEmpty,DeviceShareDataError,DeviceShareExpired,DeviceNotOnline,DeviceLiveClosed</errorCodes>
        /// <returns></returns>
        Task<JoinDeviceLiveRoomByShareResult> JoinDeviceLiveRoomByShareAsync(JoinDeviceLiveRoomByShareRequest request);

        /// <summary>
        /// 用户离开设备直播房间（无token）
        /// </summary>
        /// <param name="request">用户离开设备直播房间请求实体</param>
        /// <errorCodes>DeviceCodeIsEmpty,UniqueIdIsEmpty</errorCodes>
        /// <returns></returns>
        Task<bool> LeaveDeviceLiveRoomByShareAsync(LeaveDeviceLiveRoomByShareRequest request);

        /// <summary>
        /// 上报观看直播状态（无token）
        /// </summary>
        /// <param name="request">上报观看直播状态请求实体</param>
        /// <errorCodes>DeviceCodeIsEmpty,UniqueIdIsEmpty</errorCodes>
        /// <returns></returns>
        Task<bool> ReportLiveViewStateByShareAsync(ReportLiveViewStateByShareRequest request);

        /// <summary>
        /// 上报直播状态
        /// </summary>
        /// <param name="request">上报直播状态请求实体</param>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>设备信息接口</docSecondDirectory>  
        /// <errorCodes></errorCodes>
        /// <returns></returns>
        Task<bool> ReportLiveStateAsync(ReportLiveStateRequest request);

        /// <summary>
        /// 获取实时音视频参数
        /// </summary>
        /// <param name="request">获取实时音视频参数请求实体</param>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>设备信息接口</docSecondDirectory>  
        /// <errorCodes></errorCodes>
        /// <returns></returns>
        Task<CreateLiveRoomInfoResult> CreateLiveRoomInfoAsync(CreateLiveRoomInfoRequest request);

        // /// <summary>
        // /// 拒绝远程控制
        // /// </summary>
        // /// <param name="request"></param>
        // /// <docFirstDirectory>设备端API</docFirstDirectory>
        // /// <docSecondDirectory>远程控制接口</docSecondDirectory>  
        // /// <returns></returns>
        // Task<bool> RejectRemoteControl(RemoteControlRequest request);

        // /// <summary>
        // /// 断开远程控制
        // /// </summary>
        // /// <param name="request"></param>
        // /// <docFirstDirectory>设备端API</docFirstDirectory>
        // /// <docSecondDirectory>远程控制接口</docSecondDirectory>  
        // /// <returns></returns>
        // Task<bool> DisconnectRemoteControl(RemoteControlRequest request);

        /// <summary>
        /// 上传会诊图像
        /// </summary>
        /// <param name="request">上传会诊图像请求实体</param>
        /// <returns>上传会诊图像结果</returns>
        /// <value>true</value>
        /// <remarks>POST</remarks>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>实时会诊服务接口</docSecondDirectory>
        /// <errorCodes>ConsultationCodeIsEmpty,FileTokenIsEmpty,ConsultationNotExisted</errorCodes>
        Task<bool> UploadConsultationDataAsync(UploadConsultationDataRequest request);//todo 错误码

        /// <summary>
        /// 获取设备参数调节数据请求
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<DeviceControlParameterDataDTO> GetControlParametersAsync(GetControlParametersRequest request);

        #region Remote Maintain
        /// <summary>
        /// 申请连接/断开请求接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> ApplyRemoteConnectionAsync(ControlDeviceConnectRequest request);

        /// <summary>
        /// 设备端接受连接/断开链接请求接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> AcceptRemoteConnnectionAsync(ControlDeviceResponseRequest request);

        /// <summary>
        /// 远程控制请求接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> ApplyRemoteControlAsync(ControlDeviceParameterRequest request);

        /// <summary>
        /// 获取日志请求接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> GetRemoteLogAsync(GetRemoteLogRequest request);

        /// <summary>
        /// 设备端响应日志下载地址请求接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> ResponseRemoteLogAsync(RemoteLogResponseRequest request);

        /// <summary>
        /// 个人扫一扫添加超声机
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>UserNotExistError,UserHaveNoOrg,DeviceNotExist,HasAddedDevice</errorCodes>
        Task<bool> ScanBindDeviceAsync(ScanBindDeviceRequest request);

        /// <summary>
        /// 新增设备品牌型号画面尺寸
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>设备信息接口</docSecondDirectory>  
        /// <returns>是否操作成功</returns>
        Task<bool> ReportBrandModelOutputConfigAsync(ReportBrandModelOutputConfigRequest request);

        /// <summary>
        /// 获取设备品牌型号画面尺寸
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>设备信息接口</docSecondDirectory>  
        /// <returns></returns>
        Task<List<BrandModelOutputConfigDTO>> SyncBrandModelOutputConfigAsync(SyncBrandModelOutputConfigRequest request);

        /// <summary>
        /// 获取品牌列表
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>设备信息接口</docSecondDirectory>  
        /// <returns></returns>
        Task<List<string>> GetBrandsAsync(GetBrandsRequest request);

        /// <summary>
        /// 获取品牌型号列表
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>设备信息接口</docSecondDirectory>  
        /// <returns></returns>
        Task<List<string>> GetModelsAsync(GetModelsRequest request);

        /// <summary>
        /// 增加更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        ///  <errorCodes>NameEmpty,FileTokenIsEmpty,DeviceTypeRequired,PatchNameExistError</errorCodes>
        Task<bool> AddDevicePatchAsync(AddDevicePatchRequest request);

        /// <summary>
        /// 查询更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageResult<DevicePatchDTO>> FindDevicePatchPagesAsync(FindDevicePatchPageRequest request);

        /// <summary>
        /// 删除更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        ///  <errorCodes>PatchCodeIsEmpty</errorCodes>
        Task<bool> DeletePatchByCodeAsync(DeleteDevicePatchByCodeRequest request);

        /// <summary>
        /// 修改更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        ///  <errorCodes>NameEmpty,PatchCodeIsEmpty,PatchNoFoundError,PatchNameExistError</errorCodes>
        Task<bool> UpdateDevicePatchAsync(UpdateDevicePatchRequest request);

        /// <summary>
        ///推送或者取消设备补丁包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        ///  <errorCodes>DeviceCodeRequired,PatchCodeIsEmpty,PatchNoFoundError</errorCodes>
        Task<bool> PushDevicePatchAsync(PushDevicePatchRequest request);


        /// <summary>
        /// 根据设备编码查询可推送的更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        ///   <errorCodes>DeviceCodeRequired,DeviceNotExist</errorCodes>
        Task<PageResult<DevicePatchDTO>> FindPushDevicePatchPagesAsync(FindPushDevicePatchPageRequest request);

        /// <summary>
        ///上传补丁包下载进度
        /// </summary>
        /// <param name="request"></param>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>设备信息接口</docSecondDirectory>  
        /// <returns></returns>
        Task<bool> UploadDeviceDownloadPatchProgressToUserAsync(UploadDeviceDownloadPatchProgressToUserRequest request);

        /// <summary>
        /// 增加设备更新包版本号
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> AddDevicePatchVersionAsync(AddDevicePatchVersionRequest request);

        /// <summary>
        /// 查询设备更新包版本号
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        ///  <errorCodes>DeviceTypeEmpty</errorCodes>
        Task<List<DevicePatchVersionDTO>> FindDevicePatchVersionListAsync(FindDevicePatchListRequest request);

        /// <summary>
        /// 删除设备更新包版本号
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        ///  <errorCodes>CodeIsEmpty</errorCodes>
        Task<bool> DeleteDevicePatchVersionAsync(DeleteDevicePatchVersionRequest request);

        /// <summary>
        /// 增加打印机系统版本号
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <errorCodes>VersionInvalid,VersionIsExistError</errorCodes>
        Task<bool> AddDevicePrinterVersionAsync(AddDevicePrinterVersionRequest request);

        /// <summary>
        /// 查询打印机系统版本号
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<List<DevicePrinterVersionDTO>> FindDevicePrinterVersionListAsync(FindDevicePrinterVersionListRequest request);

        /// <summary>
        /// 删除打印机系统版本号
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <errorCodes>CodeIsEmpty</errorCodes>
        Task<bool> DeleteDevicePrinterVersionAsync(DeleteDevicePrinterVersionRequest request);

        /// <summary>
        /// 增加打印机更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        ///  <errorCodes>NameEmpty,FileTokenIsEmpty</errorCodes>
        Task<bool> AddDevicePrinterAsync(AddDevicePrinterRequest request);

        /// <summary>
        /// 查询打印机更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageResult<DevicePrinterDTO>> FindDevicePrinterPagesAsync(FindDevicePrinterPageRequest request);

        /// <summary>
        /// 删除打印机更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <errorCodes>CodeIsEmpty</errorCodes>
        Task<bool> DeletePrinterByCodeAsync(DeleteDevicePrinterByCodeRequest request);

        /// <summary>
        /// 修改打印机更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <errorCodes>NameEmpty,CodeIsEmpty,DevicePrinterExistError</errorCodes>
        Task<bool> UpdateDevicePrinterAsync(UpdateDevicePrinterRequest request);

        /// <summary>
        /// 设置设备打印机
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <errorCodes>DeviceCodeRequired,DeviceNotFind,DevicePrinterPatchNoFound</errorCodes>
        Task<bool> SetDevicePrinterAsync(SetDevicePrinterRequest request);

        /// <summary>
        /// 设备上传打印机信息
        /// </summary>
        /// <param name="request"></param>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>设备信息接口</docSecondDirectory>
        /// <returns></returns>
        /// <errorCodes>NoExistDevice,UsercodeIsEmpty</errorCodes>
        Task<bool> UploadDevicePrinterAsync(UploadDevicePrinterRequest request);

        /// <summary>
        /// 重启设备
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <errorCodes>UsercodeIsEmpty,DeviceCodeEmpty,DeviceNotOnline,DeviceCannotRestart</errorCodes>
        Task<bool> RestartDeviceAsync(RestartDeviceRequest request);

        #endregion

        /// <summary>
        /// 修改用户急诊设备
        /// </summary>
        /// <param name="request">修改用户急诊设备请求</param>
        /// <returns></returns>
        Task<bool> ModifyEmergencyDeviceCodeAsync(ModifyEmergencyDeviceCodeRequest request);

        /// <summary>
        /// 发送指令给设备
        /// </summary>
        /// <docFirstDirectory>客户端API</docFirstDirectory>
        /// <docSecondDirectory>远程维护接口</docSecondDirectory>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<string> SendCommandToDeviceAsync(SendCommandToDeviceRequest request);

        /// <summary>
        /// 发送结果给客户端
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>远程维护接口</docSecondDirectory>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<string> SendResultToClientAsync(SendResultToClientRequest request);

        /// <summary>
        /// 获取远程维护结果
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>远程维护接口</docSecondDirectory>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<string> GetResultFromServerAsync(GetResultFromServerRequest request);

        /// <summary>
        /// 断开远程连接，并通知客户端更新状态
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>远程维护接口</docSecondDirectory>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> DisconnectRemoteControl(RemoteConnectStautsRequest request);

        /// <summary>
        /// 远程维护连接心跳
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>远程维护接口</docSecondDirectory>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> RemoteConnectHeartRateAsync(RemoteConnectHeartRateRequest request);

        /// <summary>
        /// 检查要连接的超声设备是否空闲
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>远程维护接口</docSecondDirectory>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> CheckDeviceIsIdleAsync(GetDeviceRequest request);

        /// <summary>
        /// 检查用户是已经远程连接
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>远程维护接口</docSecondDirectory>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<string> CheckUserIsExistRoomIdAsync(GetDeviceRequest request);

        /// <summary>
        /// 添加用户远程连接
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>远程维护接口</docSecondDirectory>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> AddUserRemoteConnectAsync(AddUserRemoteConnectRequest request);

        /// <summary>
        /// 关闭用户正在进行的远程连接
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>远程维护接口</docSecondDirectory>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> CloseUserRemoteConnectAsync(GetDeviceRequest request);

        /// <summary>
        /// 用户取消日志下载
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>远程维护接口</docSecondDirectory>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> UserCancelLogDownloadAsync(GetDeviceRequest request);

        /// <summary>
        /// 查询用户设备连接状态列表
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>远程维护接口</docSecondDirectory>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IList<DeviceConnectStateResult>> GetDeviceStateListByTokenAsync(GetDeviceStateListRequest request);

        /// <summary>
        /// 设备取消日志下载 
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>远程维护接口</docSecondDirectory>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> DeviceCancelLogDownloadAsync(DeivceCancelLogDownloadRequest request);

        /// <summary>
        /// 设备端远程维护连接心跳
        /// </summary>
        /// <docFirstDirectory>设备端API</docFirstDirectory>
        /// <docSecondDirectory>远程维护接口</docSecondDirectory>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> DeviceRemoteConnectHeartRateAsync(RemoteConnectHeartRateRequest request);

        /// <summary>
        /// 客户端申请探头应用设置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> ApplyProbeApplicationSettingAsync(ProbeApplicationSettingRequest request);

        /// <summary>
        /// 设备端响应探头应用设置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> ResponseProbeApplicationSettingAsync(ProbeApplicationSettingResultRequest request);

        /// <summary>
        /// 客户端/设备端获取探头应用数据请求
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<ProbeApplicationSettingInfoDTO> GetProbeApplicationSettingAsync(GetControlParametersRequest request);

        /// <summary>
        /// 客户端/设备端获取探头应用数据请求
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<RemoteConnectsRequest> FindConnectingDeviceListAsync(GetDeviceRequest request);

        /// <summary>
        /// 客户端/设备端获取探头应用数据请求
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> CloseConnectingDeviceListAsync(GetDeviceRequest request);

        /// <summary>
        /// 获取设备上传打印机列表信息请求
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IList<DevicePrinterParameterDTO>> GetDevicePrintersForUploadAsync(GetDevicePrintersForUploadRequest request);

        /// <summary>
        /// 客户端获取补丁包类型集合
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IList<string>> GetDevicePatchTypeListAsync(TokenRequest request);

        /// <summary>
        /// 查询打印机驱动是否存在
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> ExistsDevicePrinterAsync(ExistsDevicePrinterRequest request);

        /// <summary>
        /// 批量新增打印机驱动
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        Task<bool> BatchInsertDevicePrintersDataAsync(MigrateDevicePrinterRequest request);

        /// <summary>
        /// 批量新增补丁包
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        Task<bool> BatchInsertDevicePatchsDataAsync(MigrateDevicePatchRequest request);

        /// <summary>
        /// 查询机构管理员机构名下所有设备列表接口
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>返回设备信息列表</returns>
        Task<PageCollection<DeviceExtendInfoDTO>> GetOrganizationDeviceListAsync(GetPersonDeviceRequest request);

        /// <summary>
        /// 获取个人绑定设备分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageCollection<SelectItemDTO>> GetPersonDeviceDropdownPageAsync(GetPersonDeviceDropdownPageRequest request);

        /// <summary>
        /// 获取设备连接列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> ExistConnectingDeviceInfoAsync(GetDeviceRequest request);

        /// <summary>
        /// 获取license信息列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IList<LicenseInfoBasicDTO>> GetLicenseInfoListAsync(TokenRequest request);

        /// <summary>
        /// 保存License信息数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> SaveLicenseInfoListAsync(SaveLicenseInfoRequest request);

        /// <summary>
        /// 保存Dongle信息数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> SaveDongleInfoListAsync(SaveDongleInfoRequest request);
    }
}