﻿using System.Threading.Tasks;
using WingInterfaceLibrary.Request.User;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Result.User;
using WingInterfaceLibrary.Internal.Request;

namespace WingInterfaceLibrary.Interface
{
    /// <summary>
    /// 登录注册服务
    /// </summary>
    /// <docFirstDirectory>客户端API</docFirstDirectory>
    /// <docSecondDirectory>个人信息接口</docSecondDirectory>
    public interface ILoginService
    {
        /// <summary>
        /// 通用登录
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>Session信息</returns>
        /// <errorCodes>VerificationCodeIllegality,UserNotRegistered,IncorrectPasswordFormat,AccountIsEmpty,AccountLengthError,SMSVerifyCodeIsEmpty,SMSVerifyCodeExpired,SMSVerifyCodeIllegality,ExceptionError,EmailVerifyCodeExpired,PhoneIncorrect,EmailIncorrect,EmailEmpty,EmailUserNotExisted,PasswordEmpty,PasswordError,IncorrectAccount,PhoneEmpty,PhoneUserNotExisted,AccountIsLocked</errorCodes>
        Task<LoginResult> CommonLoginAsync(CommonLoginRequest request);

        Task<int> CheckLoginTypeAsync(CheckLoginTypeRequest request);

        /// <summary>
        /// 通用注册
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>UserNameRegistered,PhoneRegistered,EmailRegistered,NewPasswordEmpty,AccountIsEmpty,AccountLengthError,PhoneIncorrect,VerificationCodeIncorrect,EmailIncorrect,EmailVerifyCodeExpired,VerificationCodeIllegality</errorCodes>
        Task<bool> CommonSignUpAsync(CommonSignUpRequest request);

        Task<bool> CheckSMSVerificationCodeAsync(CheckSMSVerificationCodeRequest request);

        /// <summary>
        /// 发送短信验证码
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        Task<bool> SendSMSVerificationCodeAsync(SendSMSVerificationCodeRequest request);

        /// <summary>
        /// 发送邮箱验证码
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>EmailIncorrect</errorCodes>
        Task<bool> SendEmailVerificationCodeAsync(SendEmailVerificationCodeRequest request);

        /// <summary>
        /// 检查邮箱验证码
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>EmailIncorrect,EmailVerifyCodeExpired,VerificationCodeIllegality</errorCodes>
        Task<bool> CheckEmailVerificationCodeAsync(CheckEmailVerificationCodeRequest request);

        /// <summary>
        /// 根据手机找回密码
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>VerificationCodeIllegality,UserNotRegistered</errorCodes>
        Task<bool> RetrievePasswordByPhoneAsync(RetrievePasswordByPhoneRequest request);

        /// <summary>
        /// 根据邮箱找回密码
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>EmailIncorrect,VerificationCodeIllegality,UserNotRegistered</errorCodes>
        Task<bool> RetrievePasswordByEmailAsync(RetrievePasswordByEmailRequest request);

        /// <summary>
        /// 验证账户名是否可用
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>是否可用</returns>
        /// <value>true</value>
        Task<bool> VerifyAccountAsync(VerifyAccountRequest request);

        /// <summary>
        /// 修改密码
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>UserNotExisted,PhoneIllegality,VerificationCodeIllegality,EmailIllegality,UserNameIllegality,PasswordError,UserNotRegistered,NewPasswordEmpty</errorCodes>
        Task<bool> ModifyPasswordAsync(ModifyPasswordRequest request);

        /// <summary>
        /// 生成密码
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<string> GenerateNewPasswordAsync(GenerateNewPasswordRequest request);

        /// <summary>
        /// 退出登录
        /// </summary>
        /// <param name="request">登录token信息</param>
        /// <returns>是否成功</returns>
        Task<bool> LoginOutAsync(TokenRequest request);

        /// <summary>
        /// 强制踢出
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<bool> LoginOutByAccountAsync(CommonLoginRequest request);

        /// <summary>
        /// 关联账户
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<StartAssociatedWithAccountResult> StartAssociatedWithAccountAsync(StartAssociatedWithAccountRequest request);

        /// <summary>
        /// 获取关联账户信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<GetAssociatedAccountInfoResult> GetAssociatedAccountInfoAsync(GetAssociatedAccountInfoRequest request);

        /// <summary>
        /// 解绑账户
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        // Task<ClearAssociatedWithAccountResult> ClearAssociatedWithAccountAsync(ClearAssociatedWithAccountRequest request);

        /// <summary>
        /// 获取扫码唯一码接口
        /// </summary>
        /// <returns></returns>
        Task<GetScanCodeResult> GetScanCodeAsync(GetScanCodeRequest request);

        /// <summary>
        /// 确认扫码
        /// </summary>
        /// <returns></returns>
        /// <errorCodes>UserScanInfoError</errorCodes>
        Task<bool> ConfirmScanAsync(ConfirmScanRequest request);

        /// <summary>
        /// 查询扫码确认状态
        /// </summary>
        /// <returns></returns>
        Task<CheckConfirmScanStateResult> CheckConfirmScanStateAsync(CheckConfirmScanStateRequest request);

        /// <summary>
        /// 微信通用注册
        /// </summary>
        /// <param name="request">注册请求实体</param>
        /// <returns>是否成功</returns>
        Task<LoginResult> WXCommonSignUpAsync(CommonSignUpRequest request);
    }
}
