﻿using System.Threading.Tasks;
using WingInterfaceLibrary.DTO.Record;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.Record;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.Result.Record;
using System.Collections.Generic;
using WingInterfaceLibrary.Request.QualityControl;

namespace WingInterfaceLibrary.Interface
{
    /// <summary>
    /// 检查记录服务接口
    /// </summary>
    /// <docFirstDirectory>客户端API</docFirstDirectory>
    /// <docSecondDirectory>检查记录服务接口</docSecondDirectory>
    public interface IRecordInfoService
    {
        /// <summary>
        /// 创建检查记录
        /// </summary>
        /// <param name="request">创建检查请求实体</param>
        /// <returns>是否成功</returns>
        /// <errorCodes>PatientCodeIsEmpty,DeviceCodeIsEmpty,PatientNotExistError,PatientNoSameOrg,NoExistDevice,DeviceNoSameOrg</errorCodes>
        Task<bool> CreateRecordInfoAsync(CreateRecordRequest request);


        /// <summary>
        /// 创建病人检查记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<string> CreatePatientRecordInfoAsync(CreateRecordRequest request);

        /// <summary>
        /// 客户端上传检查图像
        /// </summary>
        /// <param name="request">客户端上传检查图像请求实体</param>
        /// <returns>是否成功</returns>
        /// <errorCodes></errorCodes>
        Task<bool> CreateExamDataAsync(CreateExamDataRequest request);

        /// <summary>
        /// 分页查找检查列表
        /// </summary>
        /// <param name="request">分页查找检查列表请求实体</param>
        /// <returns>检查列表</returns>
        /// <errorCodes>PatientCodeIsEmpty,PatientNotExistError,PatientNoSameOrg</errorCodes>
        Task<PageResult<GetRecordsPageDTO>> GetRecordInfoPagesAsync(GetRecordsPageRequest request);

        /// <summary>
        /// 查询检查记录详情
        /// </summary>
        /// <param name="request">查询检查记录详情请求实体</param>
        /// <returns>检查记录详情</returns>
        /// <errorCodes>RecordCodeIsEmpty,RecordNotExistError</errorCodes>
        Task<QueryRecordResult> QueryRecordInfoAsync(QueryRecordRequest request);

        /// <summary>
        /// 查询未读检查数量
        /// </summary>
        /// <param name="request">登录人token</param>
        /// <returns>未读检查数量</returns>
        /// <value>10</value>
        /// <remarks>POST</remarks>
        Task<int> GetUnReadExamNumAsync(TokenRequest request);

        /// <summary>
        /// 动态处理检查数据接口
        /// </summary>
        /// <param name="request">动态处理检查数据请求实体</param>
        /// <returns>动态处理检查数据返回实体</returns>
        /// <errorCodes>GestationalAgeOriginIsEmpty</errorCodes>
        Task<ProcessRecordDataResult> ProcessRecordDataAsync(ProcessRecordDataRequest request);

        /// <summary>
        /// 关联检查
        /// </summary>
        /// <param name="request">关联检查i请求实体</param>
        /// <returns>返回是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>ReservationNoExistError,ExamNotExistError,RecordNoSameDevice,ExamNotRemedicalDataError</errorCodes>
        Task<bool> RelevanceRecordAsync(RelevanceRecordRequest request);

        /// <summary>删除检查记录</summary>
        /// <param name="request">删除检查记录请求实体</param>
        /// <returns>返回是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>ExamNotExistError,NoDeleteRecordFeature</errorCodes>
        Task<bool> DeleteRecordAsync(DeleteRecordRequest request);

        /// <summary>
        /// 客户端完成检查
        /// </summary>
        /// <param name="request">客户端完成检查请求实体</param>
        /// <returns>返回是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>RecordNotExistError</errorCodes>
        Task<bool> FinishRecordAsync(FinishRecordRequest request);

        /// <summary>
        /// 保存客户端上传云端测量图像信息接口
        /// </summary>
        /// <param name="request">客户端完成检查请求实体</param>
        /// <returns>返回是否成功</returns>
        /// <value>true</value>
        /// <docFirstDirectory>客户端API</docFirstDirectory>
        /// <docSecondDirectory>云端测量图像服务接口</docSecondDirectory>
        /// <errorCodes>RecordNotExistError,RemedicalMeasuredInfosIsEmpty,ExamNotRemedicalDataError,FileTokenIsEmpty,LiveConsultationNotFound</errorCodes>
        Task<bool> AddRemedicalMeasuredInfoAsync(AddRemedicalMeasuredInfoRequest request);

        /// <summary>
        /// 查询客户端保存到云端测量图像信息接口
        /// </summary>
        /// <param name="request">查询客户端保存到云端测量图像信息请求实体</param>
        /// <docFirstDirectory>客户端API</docFirstDirectory>
        /// <docSecondDirectory>云端测量图像服务接口</docSecondDirectory>
        /// <errorCodes>RecordNotExistError,LiveConsultationNotFound</errorCodes>
        Task<List<RemedicalMeasuredInfoDTO>> FindRemedicalMeasuredInfoAsync(FindRemedicalMeasuredInfoRequest request);

        /// <summary>
        /// 根据图像code查询测量图像信息接口
        /// </summary>
        /// <param name="request">查询客户端保存到云端测量图像信息请求实体</param>
        /// <docFirstDirectory>客户端API</docFirstDirectory>
        /// <docSecondDirectory>云端测量图像服务接口</docSecondDirectory>
        /// <errorCodes>RecordNotExistError,LiveConsultationNotFound</errorCodes>
        Task<List<RemedicalMeasuredInfoDTO>> FindRemedicalMeasuredInfoByRemedicalCodeAsync(FindRemedicalMeasuredInfoByRemedicalCodeRequest request);

        /// <summary>
        /// 检查是否采图空闲true为空闲
        /// </summary>
        /// <param name="request">TokenRequest</param>
        /// <returns>true</returns>
        Task<bool> CheckCollectingImgAsync(CheckCollectingImgRequest request);

        /// <summary>
        /// 开始采图
        /// </summary>
        /// <param name="request">开始采图请求实体</param>
        /// <returns>true</returns>
        /// <errorCodes>CanNotCollect,DeviceNotOnline</errorCodes>
        Task<bool> StartCollectingImgAsync(StartCollectingImgRequest request);

        /// <summary>
        /// 查询检查记录大列表（分页）
        /// </summary>
        /// <param name="request">查询检查记录大列表（分页）请求实体</param>
        /// <returns></returns>
        Task<PageResult<SimpleRecordInfoDTO>> FindRecordPagesAsync(FindRecordPagesRequest request);

        /// <summary>
        /// 大列表创建检查记录
        /// </summary>
        /// <param name="request">创建检查请求实体</param>
        /// <returns>是否成功</returns>
        /// <errorCodes>UpdatePatientInfoFail,PatientCodeIsEmpty,DeviceCodeIsEmpty,PatientNotExistError,PatientNoSameOrg,NoExistDevice,DeviceNoSameOrg</errorCodes>
        Task<bool> CreateForRecordInfoListAsync(CreateRecordNewRequest request);

        /// <summary>
        /// 大列表申请转诊
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <value>true</value>
        /// <errorCodes>ExamRecordsIsEmpty,CanNotReferral,PatientInfoIsNotValid,ExistUnCompleteScan,NoRightToReferralInThisOrg</errorCodes>
        Task<bool> ApplyReferralForRecordInfoListAsync(CreateReferralRecordNewRequest request);

        /// <summary>
        /// 大列表撤回转诊
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <value>true</value>
        /// <errorCodes>ExamRecordsIsEmpty,CanNotReferral,PatientInfoIsNotValid,ExistUnCompleteScan,NoRightToReferralInThisOrg</errorCodes>
        Task<bool> WithdrawReferralForRecordInfoListAsync(WithdrawReferralForRecordListRequest request);

        /// <summary>
        /// 查询转诊历史
        /// </summary>
        /// <param name="request">查询查询转诊历史</param>
        /// <returns>转诊历史</returns>
        Task<List<ReferralHistoryDetail>> FindReferralHistoryAsync(FindReferralHistoryRequest request);

        /// <summary>
        /// 新增图像质控
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> AddRemedicalQualityAsync(AddRemedicalQualityRequest request);

        /// <summary>
        /// 修改图像质控
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> ModifyRemedicalQualityAsync(ModifyRemedicalQualityRequest request);

        /// <summary>
        /// 删除图像质控
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> DeleteRemedicalQualityAsync(DeleteRemedicalQualityRequest request);

        /// <summary>
        /// 新增分时诊断报告质控
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> AddRecordReportQualityAsync(AddRecordReportQualityRequest request);

        /// <summary>
        /// 修改分时诊断报告质控
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> ModifyRecordReportQualityAsync(ModifyRecordReportQualityRequest request);

        /// <summary>
        /// 删除分时诊断报告质控
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> DeleteRecordReportQualityAsync(DeleteRecordReportQualityRequest request);
    }
}