using System;
using System.Collections;
using System.Collections.Generic;
using System.Threading.Tasks;
using WingInterfaceLibrary.DTO.Common;
using WingInterfaceLibrary.DTO.Device;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.IOTDevice;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.Result.Device;
using WingInterfaceLibrary.Result.IOTDevice;

namespace WingInterfaceLibrary.Interface
{
    /// <summary>
    /// IOT服务包括制氧机和呼吸机等
    /// </summary>
    /// <docFirstDirectory>客户端API</docFirstDirectory>
    /// <docSecondDirectory>IOT服务接口</docSecondDirectory>
    public interface IVinnoIOTService
    {
        /// <summary>
        /// IOT获取轮播图信息
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IList<IOTDeviceCarouselsResult>> GetCarousels(TokenRequest request);

        /// <summary>
        /// IOT获取资讯信息
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<PageResult<IOTDeviceNewsResult>> GetNews(GetIOTDeviceNewsPageRequest request);

        /// <summary>
        /// IOT获取操作指导信息
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<PageResult<IOTDeviceOperateGuidesResult>> GetOperateGuides(GetOperateGuidesPageRequest request);

        /// <summary>
        /// 获取IOT设备详细数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IOTDeviceDetailResult> GetDeviceDetail(GetIOTDeviceDetailRequest request);

        /// <summary>
        /// 获取IOT设备阶段报告
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<string> GetDeviceReport(GetIOTDeviceReportRequest request);

        /// <summary>
        /// 获取IOT设备阶段时间内有使用记录日期
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IList<DateTime>> GetDeviceUseDates(GetIOTDeviceUseDateRequest request);

        /// <summary>
        /// 获取IOT设备使用模式数据源
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IList<ModeType>> GetDeviceModeTypes(GetIOTDevicBaseRequest request);

        /// <summary>
        /// 绑定设备-用户:设备 = 1：N；设备同时只能被一个用户绑定
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IOTDeviceOperateResult> BindDevice(UserBindDeviceRequest request);

        /// <summary>
        /// 删除设备
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IOTDeviceOperateResult> DeleteDevice(DeleteDeviceInfoRequest request);

        /// <summary>
        /// 获取用户设备数据列表
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IList<UserDeviceInfoResult>> GetDevices(TokenRequest request);

        /// <summary>
        /// 获取设备首页总结数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IOTDeviceSumaryResult> Summary(GetSummaryRequest request);

        /// <summary>
        /// 获取 呼吸机|制氧机 最新版本号数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IOTDeviceVersionResult> CheckVersion(GetDeviceVersionRequest request);

        /// <summary>
        /// IOT设备属性上报
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IOTDeviceOperateResult> DeviceAttributePost(IOTDeviceAttributeEscalationRequest request);

        /// <summary>
        /// IOT设备属性查询
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<string> DeviceAttributeGet(IOTDeviceAttributeEscalationRequest request);

        /// <summary>
        /// 发布主题消息
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<string> PublishDeviceSetTopic(IOTDeviceTopicInfoRequest request);

        /// <summary>
        /// IOT设备事件上报
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IOTDeviceOperateResult> DeviceEventPost(IOTReportAttributeEscalationRequest request);

        /// <summary>
        /// IOT设备注册
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IOTDeviceOperateResult> IOTDeviceRegisterAsync(IOTDeviceRegisterRequest request);

        /// <summary>
        /// IOT设备时间同步
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IOTDeviceBasicDataResult> DeviceGetServerTimeAsync(TokenRequest request);
 
        /// <summary>
        /// 天气同步
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IOTDeviceBasicWeatherResult> GetWeatherInfoAsync(IOTWeatherRequest request);

        /// <summary>
        /// IOT设备连接云服务
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>授权响应</returns>
        /// <remarks>POST</remarks>        
        Task<ConnectResult> IOTDeviceConnectAsync(IOTDeviceConnectRequest request);

        /// <summary>
        /// 检查设备是否连接云服务
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>授权响应</returns>
        /// <remarks>POST</remarks>        
        Task<ConnectResult> CheckDeviceTokenAsync(TokenRequest request);

        /// <summary>
        /// 根据key获取字典列表
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        Task<IList<CommonDictionaryDTO>> GetCommonDictionaryInfosAsync(CommonDictionaryRequest request);

        /// <summary>
        /// 根据key获取字典信息对象
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        Task<CommonDictionaryDTO> GetCommonDictionaryDetailAsync(CommonDictionaryRequest request);

        /// <summary>
        /// 修改用户信息
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        Task<bool> ModifyUserInfoAsync(WingInterfaceLibrary.Request.User.AlterUserInfoExtendRequest request);

        /// <summary>
        /// 查询呼吸机设备数据标记列表
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        Task<IList<IOTDeviceUserListDTO>> GetIOTDeviceDataListAsync(GetIOTDeviceFileDataRequest request);
        
        /// <summary>
        /// 保存设备用户原始数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        Task<string> SaveIOTDeviceDataAsync(SaveIOTDeviceDataRequest request);

        /// <summary>
        /// 查询呼吸机设备绑定用户信息列表
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        Task<List<IOTDeviceUserListDTO>> GetIOTDeviceUserListAsync(GetIOTDeviceUsersRequest request);

        /// <summary>
        /// 保存通用设备用户信息
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        Task<bool> SaveIOTDeviceUserInfoAsync(SaveIOTDeviceUserDataRequest request);

        /// <summary>
        /// 分页查询设备用户信息
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        Task<PageResult<IOTDeviceUserDTO>> FindIOTDeviceUserPagesAsync(GetIOTDeviceUserDataPageRequest request);

        /// <summary>
        /// 分页查询设备用户数据列表
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        Task<PageResult<IOTDeviceUserInfoDTO>> FindIOTDeviceUseListPageAsync(GetIOTDeviceBindUserDataPageRequest request);

        /// <summary>
        /// 保存设备绑定用户信息
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        Task<bool> SaveIOTDeviceBindUserInfoAsync(IOTDeviceBindUserDataRequest request);        
        
        /// <summary>
        /// 保存设备用户原始数据-设备端用
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        Task<UpdateIOTDeviceDataResult> UpdateIOTDeviceDataAsync(SaveIOTDeviceDataRequest request);
        
        /// <summary>
        /// 验证设备是否已经被绑定
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        Task<List<CheckIOTDeviceBeenBoundResult>> CheckIOTDeviceBeenBoundAsync(CheckIOTDeviceBeenBoundRequest request);
        
        /// <summary>
        /// 设备阶段报告日周月年查询接口
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        Task<IOTDeviceStageReportResult> GetIOTDeviceStageReportAsync(IOTDeviceStageReportRequest request);
        
        /// <summary>
        /// 设备阶段报告趋势图查询接口
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        Task<IOTDeviceStageReportTrendChartResult> GetIOTDeviceStageReportTrendChartAsync(IOTDeviceStageReportTrendChartRequest request);
    }
}