using System.Collections.Generic;
using System.Threading.Tasks;
using WingInterfaceLibrary.DTO.RTC;
using WingInterfaceLibrary.Internal.Request;
using WingInterfaceLibrary.Rtc;

namespace WingInterfaceLibrary.Internal.Interface
{
    /// <summary>
    /// RTC服务
    /// </summary>
    public interface IWingRtcService
    {
        /// <summary>
        /// 获取roomID
        /// </summary>
        /// <param name="request"></param>
        /// <returns>roomID</returns>
        /// <show>false</show>
        Task<GetRoomIdResult> GetRoomIdAsync(GetRoomIdRequest request);
        /// <summary>
        /// 获取roomUrl
        /// </summary>
        /// <param name="request"></param>
        /// <returns>url</returns>
        /// <show>false</show>
        Task<GenerateRoomUrlResult> GenerateRoomUrlAsync(GenerateRoomUrlRequest request);
        /// <summary>
        /// 获取rtc配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns>rtc配置</returns>
        /// <show>false</show>
        Task<GetRtcSettingResult> GetRtcSettingAsync(GetRtcSettingRequest request);
        /// <summary>
        /// 获取UserSign
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<GetUserSignResult> GetUserSignAsync(GetUserSignRequest request);

        #region Live Room Manager
        /// <summary>
        /// 创建或更新直播间，并落库 
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>直播房间基础信息</returns>
        /// <show>false</show>
        Task<LiveRoomInfoDTO> AddOrModifyLiveRoomAsync(SaveLiveRoomRequest request);

        /// <summary>
        /// 获取或创建直播间     
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>直播房间基础信息</returns>
        /// <show>false</show>
        Task<LiveRoomInfoDTO> GetLiveRoomInfoAsync(BaseLiveRoomRequest request);

        /// <summary>
        /// 刷新成员信息
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>true</returns>
        /// <show>false</show>
        Task<bool> RefreshMemberInfosByTokenAsync(UpdateRoomMemberInfoRequest request);

        /// <summary>
        /// 开始直播，包含通知
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>true</returns>
        /// <show>false</show>
        Task<LiveRoomInfoDTO> InitiateLiveAsync(InitiateLiveRequest request);

        /// <summary>
        /// 接受直播中邀请
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>true</returns>
        /// <show>false</show>
        Task<LiveRoomInfoDTO> AcceptInLiveAsync(SetLiveParamsRequest request);

        /// <summary>
        /// 直播心跳，进入房间
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>true</returns>
        /// <show>false</show>
        Task<bool> HeartRateJoinAsync(HeartRateCheckRequest request);

        /// <summary>
        /// 直播心跳，网络质量不佳
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>true</returns>
        /// <show>false</show>
        Task<bool> HeartRateNetworkErrAsync(HeartRateCheckRequest request);

        /// <summary>
        /// 直播心跳，离开房间
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>true</returns>
        /// <show>false</show>
        Task<LiveRoomInfoDTO> HeartRateLeaveAsync(HeartRateLeaveRequest request);

        /// <summary>
        /// 离开房间(含结束直播)
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>true</returns>
        /// <show>false</show>
        Task<LiveRoomInfoDTO> LeaveRoomAsync(LeaveRoomRequest request);

        /// <summary>
        /// 开启关闭静音
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>true</returns>
        /// <show>false</show>
        Task<bool> ChangeMuteStateAsync(SetLiveParamsRequest request);

        /// <summary>
        /// 开启关闭视频
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>true</returns>
        /// <show>false</show>
        Task<bool> ChangeVideoOpenStateAsync(SetLiveParamsRequest request);

        /// <summary>
        /// 更改调参状态
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>true</returns>
        /// <show>false</show>
        Task<bool> ChangeControllingParameterStateAsync(SetLiveParamsRequest request);

        /// <summary>
        /// 修改直播项目状态
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>true</returns>
        /// <show>false</show>
        Task<bool> ChangeLiveStatusAsync(ChangeLiveStatusRequest request);

        /// <summary>
        /// 根据RTC房间号查询房间集合
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>房间列表</returns>
        /// <show>false</show>
        Task<List<LiveRoomInfoDTO>> GetLiveRoomsByRtcRoomIdAsync(GetLiveRoomsByRtcRoomIdRequest request);

        /// <summary>
        /// 根据deviceCode或者userCode获取room信息
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>房间列表</returns>
        /// <show>false</show>
        Task<LiveRoomInfoDTO> GetLiveRoomByCodeAsync(FindRoomByCodeRequest request);

        /// <summary>
        /// 根据设备编码查询房间集合
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>房间列表</returns>
        /// <show>false</show>
        Task<List<LiveRoomInfoDTO>> GetLiveRoomsByDeviceCodeAsync(GetLiveRoomsByDeviceCodeRequest request);

        /// <summary>
        /// 发送通知
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>false</returns>
        /// <show>false</show>
        Task<bool> BroadcastRoomNotificationAsync(BroadcastRoomNotificationRequest request);

        /// <summary>
        /// 开启关闭共享信息
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>true</returns>
        /// <show>false</show>
        Task<bool> ChangedShareInfoAsync(ChangedShareInfoRequest request);

        #endregion
    }
}